class Error:
    none_or_invalid_attribute = "main attributes should have value."
    unacceptable_json = "json input has unacceptable format."
    unacceptable_object_type = "object has unacceptable type"
