class BankExtTypes:
    bank_money_transfer = "BankMoneyTransfer"
    bank_remain = "BankRemain"
    bank_transaction = "BankTransaction"
    deprecated_receipt = "DeprecatedReceipt"
    deprecated_receipt_message = "DepricatedReceiptMessage"
    receipt_message = "ReceiptMessage"
