class RequestType:
    get_file_download_url = "GetFileDownloadUrl"
    get_file_upload_url = "GetFileUploadUrl"

    create_group = "CreateGroup"
    get_group_api_struct = "GetGroupApiStruct"
    invite_user = "InviteUser"

    send_message = "SendMessage"
    edit_message = "EditMessage"

    get_difference = "GetDifference"
    get_last_seq = "GetLastSeq"
