class ValueType:
    string_val = "StringVal"
    int32_val = "Int32Val"
    int64_val = "Int64Val"
    double_val = "DoubleVal"
    array_val = "ArrayVal"
    map_value = "MapValue"
    boolean_value = "BooleanValue"
