
__title__ = 'balebot'
__description__ = 'Python framework for Bale messenger Bot API'
__url__ = 'https://github.com/balemessenger/bale-bot-python'
__version__ = '1.6.12'
__author__ = 'bale'
__author_email__ = 'balebot@elenoon.ir'
__license__ = 'Apache 2.0'
