class ServiceType:
    Messaging = 'messaging'
    Files = 'files'
    Stickers = 'stickers'
    Users = 'users'
    KeyValue = 'keyvalue'
    Bots = 'bots'
    WebHooks = 'webhooks'
    Monitor = 'monitor'
    Groups = "groups"
    SequenceUpdate = "sequence-update"
    banks = "banks"
